import ArrayLastValue from './ArrayLastValue.js'
import assert from './assert.js'
import QcQueueGroups from './QcQueueGroups.js'
import AnimatedSet from './AnimatedSet.js'
import qcSkipForward from './qcSkipForward.js'
import qcSkipBack from './qcSkipBack.js'
import QcQueueVolume from './QcQueueVolume.js'
import QcQueueIsPlaying from './QcQueueIsPlaying.js'
import CollabPosition from './CollabPosition.js'
import qcQueueMap from './qcQueueMap.js'
import QcButton from './QcButton.js'
import qcsCollabMap from './qcsCollabMap.js'
import QcMediaSubtitle from './QcMediaSubtitle.js'
import QcMediaTitle from './QcMediaTitle.js'
import QcMainLiContextButton from './QcMainLiContextButton.js'
import QcMainLi from './QcMainLi.js'
import QcTitleBarForm from './QcTitleBarForm.js'
import QcTitleBar from './QcTitleBar.js'
import QcMainUl from './QcMainUl.js'
import QcMain from './QcMain.js'
import QcTitleBarInput from './QcTitleBarInput.js'
import animators from './animators.js'

const PlaylistGroupMainLiRow = (playlist, tracksInGroup, expandedPlaylists, liIsFirst) => {
	const row = document.createElement('div')
	row.classList.add('mainLiRow')
	if (!liIsFirst) {
		row.appendChild((() => {
			const removeButton = QcButton()
			removeButton.addEventListener('click', () => {
				for (const track of tracksInGroup) {
					qcsCollabMap.delete('media-' + track + '-playlists-' + playlist + '-queueInfo')
				}
			})
			removeButton.appendChild(document.createTextNode('❌'))
			return removeButton
		})())
	}
	row.appendChild((() => {
		const expandButton = QcButton()
		expandButton.addEventListener('click', () => {
			if (expandedPlaylists.has(playlist)) {
				expandedPlaylists.delete(playlist)
			} else {
				expandedPlaylists.add(playlist)
			}
		})
		animators.set(expandButton, () => {
			expandButton.textContent = expandedPlaylists.has(playlist) ? '➖' : '➕'
		})
		return expandButton
	})())
	row.appendChild((() => {
		const contextButton = QcMainLiContextButton()
		contextButton.appendChild(document.createTextNode(QcMediaTitle(playlist)))
		const subtitle = QcMediaSubtitle(playlist)
		if (subtitle !== '') {
			contextButton.appendChild(document.createTextNode(' '))
			contextButton.appendChild((() => {
				const subtitleSpan = document.createElement('span')
				subtitleSpan.classList.add('mainLiSubtitleSpan')
				subtitleSpan.appendChild(document.createTextNode('(' + subtitle + ')'))
				return subtitleSpan
			})())
		}
		return contextButton
	})())
	if (!liIsFirst) {
		row.appendChild((() => {
			const moveDownButton = QcButton()
			moveDownButton.addEventListener('click', () => {
				const groups = QcQueueGroups()
				const groupIndexOrNegativeOne = groups.findIndex((group) => {
					const firstValue = qcQueueMap.get(group[0])
					return firstValue.playlist === playlist && firstValue.track === tracksInGroup[0]
				})
				if (groupIndexOrNegativeOne !== -1) {
					const groupIndex = groupIndexOrNegativeOne
					if (groupIndex !== groups.length - 1) {
						let afterPosition = ArrayLastValue(groups[groupIndex + 1])
						const beforePositionOrNull = groupIndex === groups.length - 2 ? null : groups[groupIndex + 2][0]
						for (const track of groups[groupIndex].map((position) => {
							return qcQueueMap.get(position).track
						})) {
							const newPosition = CollabPosition(afterPosition, beforePositionOrNull)
							qcsCollabMap.set(
								'media-' + track + '-playlists-' + playlist + '-queueInfo',
								JSON.stringify({position: newPosition}),
							)
							afterPosition = newPosition
						}
					}
				}
			})
			moveDownButton.appendChild(document.createTextNode('⬇️'))
			return moveDownButton
		})())
		row.appendChild((() => {
			const moveUpButton = QcButton()
			moveUpButton.addEventListener('click', () => {
				const groups = QcQueueGroups()
				const groupIndexOrNegativeOne = groups.findIndex((group) => {
					const firstValue = qcQueueMap.get(group[0])
					return firstValue.playlist === playlist && firstValue.track === tracksInGroup[0]
				})
				if (groupIndexOrNegativeOne !== -1) {
					const groupIndex = groupIndexOrNegativeOne
					if (groupIndex !== 0) {
						let afterPositionOrNull
						let beforePosition
						if (groupIndex === 1 && groups[0].length > 1) {
							afterPositionOrNull = groups[0][0]
							beforePosition = groups[0][1]
						} else {
							afterPositionOrNull = groupIndex === 1 ? null : ArrayLastValue(groups[groupIndex - 2])
							beforePosition = groups[groupIndex - 1][0]
						}
						for (const track of groups[groupIndex].map((position) => {
							return qcQueueMap.get(position).track
						})) {
							const newPosition = CollabPosition(afterPositionOrNull, beforePosition)
							qcsCollabMap.set(
								'media-' + track + '-playlists-' + playlist + '-queueInfo',
								JSON.stringify({position: newPosition}),
							)
							afterPositionOrNull = newPosition
						}
					}
				}
			})
			moveUpButton.appendChild(document.createTextNode('⬆️'))
			return moveUpButton
		})())
	}
	return row
}
const PlaylistGroupMainLiTracksUl = (playlist, tracksInGroup, expandedPlaylists, liIsFirst) => {
	const tracksUl = document.createElement('ul')
	tracksUl.classList.add('mainLiUl')
	animators.set(tracksUl, () => {
		tracksUl.style.display = expandedPlaylists.has(playlist) ? '' : 'none'
	})
	for (const track of (liIsFirst ? tracksInGroup.slice(1) : tracksInGroup)) {
		const queueInfoKey = 'media-' + track + '-playlists-' + playlist + '-queueInfo'
		tracksUl.appendChild((() => {
			const trackLi = QcMainLi()
			trackLi.appendChild((() => {
				const row = document.createElement('div')
				row.classList.add('mainLiRow')
				row.appendChild((() => {
					const removeButton = QcButton()
					removeButton.addEventListener('click', () => {
						qcsCollabMap.delete(queueInfoKey)
					})
					removeButton.appendChild(document.createTextNode('❌'))
					return removeButton
				})())
				row.appendChild((() => {
					const contextButton = QcMainLiContextButton()
					contextButton.appendChild(document.createTextNode(QcMediaTitle(track)))
					const subtitle = QcMediaSubtitle(track)
					if (subtitle !== '') {
						contextButton.appendChild(document.createTextNode(' '))
						contextButton.appendChild((() => {
							const subtitleSpan = document.createElement('span')
							subtitleSpan.classList.add('mainLiSubtitleSpan')
							subtitleSpan.appendChild(document.createTextNode('(' + subtitle + ')'))
							return subtitleSpan
						})())
					}
					return contextButton
				})())
				// row.appendChild((() => {
				// 	const moveDownButton = QcButton()
				// 	moveDownButton.addEventListener('click', () => {
				// 		const queueInfoAsJsonOrEmptyString = qcsCollabMap.get(queueInfoKey)
				// 		if (queueInfoAsJsonOrEmptyString !== '') {
				// 			const position = JSON.parse(queueInfoAsJsonOrEmptyString).position
				// 			for (const group of QcQueueGroups()) {
				// 				const indexInGroupOrNegativeOne = group.indexOf(position)
				// 				if (indexInGroupOrNegativeOne !== -1) {
				// 					const indexInGroup = indexInGroupOrNegativeOne
				// 					if (indexInGroup !== group.length - 1) {
				// 						// broken b/c null shouldnt be null necessarily
				// 						// qcsCollabMap.set(queueInfoKey, JSON.stringify({position: CollabPosition(group[indexInGroup + 1], indexInGroup === group.length - 2 ? null : group[indexInGroup + 2])}))
				// 					}
				// 					break
				// 				}
				// 			}
				// 		}
				// 	})
				// 	moveDownButton.appendChild(document.createTextNode('⬇️'))
				// 	return moveDownButton
				// })())
				// row.appendChild((() => {
				// 	const moveUpButton = QcButton()
				// 	moveUpButton.addEventListener('click', () => {
				// 		const queueInfoAsJsonOrEmptyString = qcsCollabMap.get(queueInfoKey)
				// 		if (queueInfoAsJsonOrEmptyString !== '') {
				// 			const position = JSON.parse(queueInfoAsJsonOrEmptyString).position
				// 			for (const group of QcQueueGroups()) {
				// 				const indexInGroupOrNegativeOne = group.indexOf(position)
				// 				if (indexInGroupOrNegativeOne !== -1) {
				// 					const indexInGroup = indexInGroupOrNegativeOne
				// 					if (indexInGroup > (liIsFirst ? 1 : 0)) {
				// 						// broken b/c null shouldnt be null necessarily
				// 						// qcsCollabMap.set(queueInfoKey, JSON.stringify({position: CollabPosition(indexInGroup === 1 ? null : group[indexInGroup - 2], group[indexInGroup - 1])}))
				// 					}
				// 					break
				// 				}
				// 			}
				// 		}
				// 	})
				// 	moveUpButton.appendChild(document.createTextNode('⬆️'))
				// 	return moveUpButton
				// })())
				return row
			})())
			return trackLi
		})())
	}
	return tracksUl
}
const QcQueuePage = (tabAfterTick) => {
	const page = {
		titleBar: (() => {
			const bar = QcTitleBar()
			bar.appendChild((() => {
				const form = QcTitleBarForm()
				form.appendChild((() => {
					const input = QcTitleBarInput()
					input.disabled = true
					return input
				})())
				return form
			})())
			return bar
		})(),
		main: (() => {
			const main = QcMain()
			main.appendChild((() => {
				const ul = QcMainUl()
				let animatedGroupsOrNull = null
				const expandedPlaylists = new AnimatedSet()
				animators.set(ul, () => {
					const groups = QcQueueGroups()
					if (animatedGroupsOrNull !== groups) {
						if (groups.length) {
							const children = [(() => {
								const tracksInGroup = groups[0].map((position) => {
									return qcQueueMap.get(position).track
								})
								const playlistOrUndefined = qcQueueMap.get(groups[0][0]).playlist
								const firstTrack = tracksInGroup[0]
								const li = QcMainLi()
								li.appendChild((() => {
									const row = document.createElement('div')
									row.classList.add('mainLiRow')
									row.appendChild((() => {
										const removeButton = QcButton()
										removeButton.addEventListener('click', () => {
											if (playlistOrUndefined === undefined) {
												qcsCollabMap.delete('media-' + firstTrack + '-queueInfo')
											} else {
												for (const track of tracksInGroup) {
													qcsCollabMap.delete('media-' + track + '-playlists-' + playlistOrUndefined + '-queueInfo')
												}
											}
										})
										removeButton.appendChild(document.createTextNode('❌'))
										return removeButton
									})())
									row.appendChild((() => {
										const contextButton = QcMainLiContextButton()
										contextButton.classList.add('queueCurrentLiContextButton')
										contextButton.appendChild(document.createTextNode(QcMediaTitle(firstTrack)))
										const subtitle = QcMediaSubtitle(firstTrack)
										if (subtitle !== '') {
											contextButton.appendChild(document.createTextNode(' '))
											contextButton.appendChild((() => {
												const subtitleSpan = document.createElement('span')
												subtitleSpan.classList.add('mainLiSubtitleSpan')
												subtitleSpan.appendChild(document.createTextNode('(' + subtitle + ')'))
												return subtitleSpan
											})())
										}
										return contextButton
									})())
									row.appendChild((() => {
										const moveDownButton = QcButton()
										moveDownButton.addEventListener('click', () => {
											const groups = QcQueueGroups()
											const groupIndexOrNegativeOne = groups.findIndex((group) => {
												const firstValue = qcQueueMap.get(group[0])
												return firstValue.playlist === playlistOrUndefined && firstValue.track === firstTrack
											})
											if (groupIndexOrNegativeOne !== -1) {
												const groupIndex = groupIndexOrNegativeOne
												if (groupIndex !== groups.length - 1) {
													let afterPosition = ArrayLastValue(groups[groupIndex + 1])
													const beforePositionOrNull = groupIndex === groups.length - 2 ? null : groups[groupIndex + 2][0]
													if (playlistOrUndefined === undefined) {
														qcsCollabMap.set(
															'media-' + firstTrack + '-queueInfo',
															JSON.stringify({position: CollabPosition(afterPosition, beforePositionOrNull)}),
														)
													} else {
														const playlist = playlistOrUndefined
														for (const track of groups[groupIndex].map((position) => {
															return qcQueueMap.get(position).track
														})) {
															const newPosition = CollabPosition(afterPosition, beforePositionOrNull)
															qcsCollabMap.set(
																'media-' + track + '-playlists-' + playlist + '-queueInfo',
																JSON.stringify({position: newPosition}),
															)
															afterPosition = newPosition
														}
													}
												}
											}
										})
										moveDownButton.appendChild(document.createTextNode('⬇️'))
										return moveDownButton
									})())
									row.appendChild((() => {
										const moveUpButton = QcButton()
										moveUpButton.appendChild(document.createTextNode('⬆️'))
										return moveUpButton
									})())
									return row
								})())
								li.appendChild((() => {
									const row = document.createElement('div')
									row.classList.add('mainLiRow')
									row.classList.add('queueCurrentLiSkipRow')
									row.appendChild((() => {
										const skipBackButton = QcButton()
										skipBackButton.classList.add('queueCurrentLiSkipBackButton')
										skipBackButton.addEventListener('click', () => {
											qcSkipBack()
										})
										skipBackButton.appendChild(document.createTextNode('⏮️'))
										return skipBackButton
									})())
									row.appendChild((() => {
										const playButton = QcButton()
										playButton.classList.add('queueCurrentLiPlayButton')
										playButton.addEventListener('click', () => {
											QcQueueIsPlaying.set(!QcQueueIsPlaying())
										})
										animators.set(playButton, () => {
											playButton.textContent = QcQueueIsPlaying() ? '⏸️' : '▶️'
										})
										return playButton
									})())
									row.appendChild((() => {
										const skipForwardButton = QcButton()
										skipForwardButton.classList.add('queueCurrentLiSkipForwardButton')
										skipForwardButton.addEventListener('click', () => {
											qcSkipForward()
										})
										skipForwardButton.appendChild(document.createTextNode('⏭️'))
										return skipForwardButton
									})())
									return row
								})())
								li.appendChild((() => {
									const row = document.createElement('div')
									row.classList.add('mainLiRow')
									row.classList.add('queueCurrentLiVolumeSliderRow')
									row.appendChild((() => {
										const volumeMutedIcon = QcButton()
										volumeMutedIcon.textContent = '🔇'
										return volumeMutedIcon
									})())
									row.appendChild((() => {
										const volumeSlider = document.createElement('input')
										volumeSlider.classList.add('queueCurrentLiVolumeSlider')
										volumeSlider.type = 'range'
										volumeSlider.min = 0
										volumeSlider.max = 1
										volumeSlider.step = 'any'
										volumeSlider.addEventListener('input', () => {
											QcQueueVolume.set(JSON.parse(volumeSlider.value))
										})
										animators.set(volumeSlider, () => {
											volumeSlider.value = QcQueueVolume().toString()
										})
										return volumeSlider
									})())
									row.appendChild((() => {
										const volumeMaxIcon = QcButton()
										volumeMaxIcon.textContent = '🔊'
										return volumeMaxIcon
									})())
									return row
								})())
								if (groups[0].length > 1) {
									assert(playlistOrUndefined !== undefined)
									li.appendChild(PlaylistGroupMainLiRow(playlistOrUndefined, tracksInGroup, expandedPlaylists, true))
									li.appendChild(PlaylistGroupMainLiTracksUl(playlistOrUndefined, tracksInGroup, expandedPlaylists, true))
								}
								return li
							})()]
							for (let i = 1; i < groups.length; i++) {
								const playlistOrUndefined = qcQueueMap.get(groups[i][0]).playlist
								if (playlistOrUndefined === undefined) {
									assert(groups[i].length === 1)
									const track = qcQueueMap.get(groups[i][0]).track
									const queueInfoKey = 'media-' + track + '-queueInfo'
									children.push((() => {
										const li = QcMainLi()
										li.appendChild((() => {
											const row = document.createElement('div')
											row.classList.add('mainLiRow')
											row.appendChild((() => {
												const removeButton = QcButton()
												removeButton.addEventListener('click', () => {
													qcsCollabMap.delete(queueInfoKey)
												})
												removeButton.appendChild(document.createTextNode('❌'))
												return removeButton
											})())
											row.appendChild((() => {
												const contextButton = QcMainLiContextButton()
												contextButton.appendChild(document.createTextNode(QcMediaTitle(track)))
												const subtitle = QcMediaSubtitle(track)
												if (subtitle !== '') {
													contextButton.appendChild(document.createTextNode(' '))
													contextButton.appendChild((() => {
														const subtitleSpan = document.createElement('span')
														subtitleSpan.classList.add('mainLiSubtitleSpan')
														subtitleSpan.appendChild(document.createTextNode('(' + subtitle + ')'))
														return subtitleSpan
													})())
												}
												return contextButton
											})())
											row.appendChild((() => {
												const moveDownButton = QcButton()
												moveDownButton.addEventListener('click', () => {
													const groups = QcQueueGroups()
													const groupIndexOrNegativeOne = groups.findIndex((group) => {
														const firstValue = qcQueueMap.get(group[0])
														return firstValue.playlist === undefined && firstValue.track === track
													})
													if (groupIndexOrNegativeOne !== -1) {
														const groupIndex = groupIndexOrNegativeOne
														if (groupIndex !== groups.length - 1) {
															const afterPosition = ArrayLastValue(groups[groupIndex + 1])
															const beforePositionOrNull = groupIndex === groups.length - 2 ? null : groups[groupIndex + 2][0]
															qcsCollabMap.set(
																'media-' + track + '-queueInfo',
																JSON.stringify({position: CollabPosition(afterPosition, beforePositionOrNull)}),
															)
														}
													}
												})
												moveDownButton.appendChild(document.createTextNode('⬇️'))
												return moveDownButton
											})())
											row.appendChild((() => {
												const moveUpButton = QcButton()
												moveUpButton.addEventListener('click', () => {
													const groups = QcQueueGroups()
													const groupIndexOrNegativeOne = groups.findIndex((group) => {
														const firstValue = qcQueueMap.get(group[0])
														return firstValue.playlist === undefined && firstValue.track === track
													})
													if (groupIndexOrNegativeOne !== -1) {
														const groupIndex = groupIndexOrNegativeOne
														if (groupIndex !== 0) {
															let afterPositionOrNull
															let beforePosition
															if (groupIndex === 1 && groups[0].length > 1) {
																afterPositionOrNull = groups[0][0]
																beforePosition = groups[0][1]
															} else {
																afterPositionOrNull = groupIndex === 1 ? null : ArrayLastValue(groups[groupIndex - 2])
																beforePosition = groups[groupIndex - 1][0]
															}
															qcsCollabMap.set(
																'media-' + track + '-queueInfo',
																JSON.stringify({position: CollabPosition(afterPositionOrNull, beforePosition)}),
															)
														}
													}
												})
												moveUpButton.appendChild(document.createTextNode('⬆️'))
												return moveUpButton
											})())
											return row
										})())
										return li
									})())

								} else {
									const playlist = playlistOrUndefined
									const tracksInGroup = groups[i].map((position) => {
										return qcQueueMap.get(position).track
									})
									children.push((() => {
										const li = QcMainLi()
										li.appendChild(PlaylistGroupMainLiRow(playlist, tracksInGroup, expandedPlaylists, false))
										li.appendChild(PlaylistGroupMainLiTracksUl(playlist, tracksInGroup, expandedPlaylists, false))
										return li
									})())
								}
							}
							ul.replaceChildren(...children)
						} else {
							ul.replaceChildren((() => {
								const li = QcMainLi()
								li.appendChild((() => {
									const row = document.createElement('div')
									row.classList.add('mainLiRow')
									row.appendChild((() => {
										const removeButton = QcButton()
										removeButton.appendChild(document.createTextNode('❌'))
										return removeButton
									})())
									row.appendChild((() => {
										const contextButton = QcMainLiContextButton()
										contextButton.classList.add('queueCurrentLiContextButton')
										contextButton.appendChild(document.createTextNode('Queue is empty'))
										return contextButton
									})())
									row.appendChild((() => {
										const moveDownButton = QcButton()
										moveDownButton.appendChild(document.createTextNode('⬇️'))
										return moveDownButton
									})())
									row.appendChild((() => {
										const moveUpButton = QcButton()
										moveUpButton.appendChild(document.createTextNode('⬆️'))
										return moveUpButton
									})())
									return row
								})())
								li.appendChild((() => {
									const row = document.createElement('div')
									row.classList.add('mainLiRow')
									row.classList.add('queueCurrentLiSkipRow')
									row.appendChild((() => {
										const skipBackButton = QcButton()
										skipBackButton.classList.add('queueCurrentLiSkipBackButton')
										skipBackButton.addEventListener('click', () => {
											qcSkipBack()
										})
										skipBackButton.appendChild(document.createTextNode('⏮️'))
										return skipBackButton
									})())
									row.appendChild((() => {
										const playButton = QcButton()
										playButton.classList.add('queueCurrentLiPlayButton')
										playButton.textContent = '▶️'
										return playButton
									})())
									row.appendChild((() => {
										const skipForwardButton = QcButton()
										skipForwardButton.classList.add('queueCurrentLiSkipForwardButton')
										skipForwardButton.appendChild(document.createTextNode('⏭️'))
										return skipForwardButton
									})())
									return row
								})())
								li.appendChild((() => {
									const row = document.createElement('div')
									row.classList.add('mainLiRow')
									row.classList.add('queueCurrentLiVolumeSliderRow')
									row.appendChild((() => {
										const volumeMutedIcon = QcButton()
										volumeMutedIcon.textContent = '🔇'
										return volumeMutedIcon
									})())
									row.appendChild((() => {
										const volumeSlider = document.createElement('input')
										volumeSlider.classList.add('queueCurrentLiVolumeSlider')
										volumeSlider.type = 'range'
										volumeSlider.min = 0
										volumeSlider.max = 1
										volumeSlider.step = 'any'
										volumeSlider.addEventListener('input', () => {
											QcQueueVolume.set(JSON.parse(volumeSlider.value))
										})
										animators.set(volumeSlider, () => {
											volumeSlider.value = QcQueueVolume().toString()
										})
										return volumeSlider
									})())
									row.appendChild((() => {
										const volumeMaxIcon = QcButton()
										volumeMaxIcon.textContent = '🔊'
										return volumeMaxIcon
									})())
									return row
								})())
								return li
							})())
						}
						animatedGroupsOrNull = groups
					}
				})
				return ul
			})())
			return () => {
				return main
			}
		})(),
	}
	return page
}

export default QcQueuePage
