import assert from './assert.js'
import http from 'node:http'
import HttpResAsJsonValue from './HttpResAsJsonValue.js'
import HttpsGetRes from './HttpsGetRes.js'
import IsStringId from './IsStringId.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'

const QsYtAudioResOrNull = async (uri) => {
	let resOrNull = null
	let threw = false
	let throweeIfThrew
	try {
		resOrNull = await HttpsGetRes(uri)
		assert(resOrNull.statusCode === 200)
		assert(resOrNull.headers['content-encoding'] === undefined)
		const contentType = resOrNull.headers['content-type']
		assert(IsStringId(contentType))
		assert(contentType !== '')
		http.validateHeaderValue('Content-Type', contentType)
	} catch (throwee) {
		threw = true
		throweeIfThrew = throwee
	}
	qsWriteDebuggingInfo('qsYtAudioResOrNullInfo', {
		resOrNull: resOrNull === null ? null : HttpResAsJsonValue(resOrNull),
		threw: threw,
		throweeIfThrew: threw ? ThroweeAsJsonValue(throweeIfThrew) : null,
		uri: uri,
	})
	if (threw) {
		if (resOrNull !== null) {
			resOrNull.destroy()
		}
		return null
	}
	return resOrNull
}

export default QsYtAudioResOrNull
