import bufferHttpResBody from './bufferHttpResBody.js'
import HttpResAsJsonValue from './HttpResAsJsonValue.js'
import HttpsGetRes from './HttpsGetRes.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import StringId from './StringId.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'
import Ungzipped from './Ungzipped.js'
import YtInitialData from './YtInitialData.js'
import YtTextAsStringId from './YtTextAsStringId.js'

const QsYtChannelReleaseResultsOrNull = async (channel, channelTitle) => {
	let resOrNull = null
	let resBodyOrNull = null
	const results = []
	let threw = false
	let throweeIfThrew
	try {
		resOrNull = await HttpsGetRes(
			'https://www.youtube.com/channel/' + encodeURIComponent(channel) + '/releases',
			{headers: {'Accept-Encoding': 'gzip', 'Accept-Language': 'en-US'}},
		)
		resBodyOrNull = await bufferHttpResBody(resOrNull)
		for (const tab of (
			YtInitialData((await Ungzipped(resBodyOrNull)).toString())
			.contents
			.twoColumnBrowseResultsRenderer
			.tabs
		)) {
			if (tab.tabRenderer !== undefined && tab.tabRenderer.content !== undefined) {
				for (const content of tab.tabRenderer.content.richGridRenderer.contents) {
					if (content.richItemRenderer !== undefined) {
						const playlistRenderer = content.richItemRenderer.content.playlistRenderer
						results.push({
							kind: 'playlist',
							subtitleOrNull: channelTitle,
							title: YtTextAsStringId(playlistRenderer.title),
							value: StringId(playlistRenderer.playlistId),
						})
					}
				}
			}
		}
	} catch (throwee) {
		threw = true
		throweeIfThrew = throwee
	}
	qsWriteDebuggingInfo('qsYtChannelReleaseResultsOrNullInfo', {
		channel: channel,
		channelTitle: channelTitle,
		resBodyOrNull: resBodyOrNull === null ? null : resBodyOrNull.toString('base64'),
		resOrNull: resOrNull === null ? null : HttpResAsJsonValue(resOrNull),
		threw: threw,
		throweeIfThrew: threw ? ThroweeAsJsonValue(throweeIfThrew) : null,
	})
	return threw ? null : results
}

export default QsYtChannelReleaseResultsOrNull
