import bufferHttpResBody from './bufferHttpResBody.js'
import HttpResAsJsonValue from './HttpResAsJsonValue.js'
import HttpsGetRes from './HttpsGetRes.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import RelativeUriAsUrl from './RelativeUriAsUrl.js'
import StringId from './StringId.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'
import Ungzipped from './Ungzipped.js'
import YtInitialData from './YtInitialData.js'
import YtTextAsStringId from './YtTextAsStringId.js'

const QsYtChannelResultsOrNull = async (channel, channelTitle) => {
	let resOrNull = null
	let resBodyOrNull = null
	const results = []
	let threw = false
	let throweeIfThrew
	try {
		resOrNull = await HttpsGetRes(
			'https://www.youtube.com/channel/' + encodeURIComponent(channel) + '/playlists',
			{headers: {'Accept-Encoding': 'gzip', 'Accept-Language': 'en-US'}},
		)
		resBodyOrNull = await bufferHttpResBody(resOrNull)
		const tabs = (
			YtInitialData((await Ungzipped(resBodyOrNull)).toString())
			.contents
			.twoColumnBrowseResultsRenderer
			.tabs
		)
		if (tabs.some((tab) => {
			if (tab.tabRenderer === undefined) {
				return false
			}
			const splitPathname = RelativeUriAsUrl((
				tab
				.tabRenderer
				.endpoint
				.commandMetadata
				.webCommandMetadata
				.url
			)).pathname.split('/')
			return splitPathname[splitPathname.length - 1] === 'videos'
		})) {
			results.push({
				kind: 'channelRecentVideos',
				title: 'Recent videos',
				subtitleOrNull: channelTitle,
				value: channel,
			})
		}
		for (const tab of tabs) {
			if (tab.tabRenderer !== undefined) {
				const splitPathname = RelativeUriAsUrl((
					tab
					.tabRenderer
					.endpoint
					.commandMetadata
					.webCommandMetadata
					.url
				)).pathname.split('/')
				if (splitPathname[splitPathname.length - 1] === 'playlists') {
					const sectionListRenderer = tab.tabRenderer.content.sectionListRenderer
					for (const sectionListContent of sectionListRenderer.contents) {
						for (const itemSectionContent of sectionListContent.itemSectionRenderer.contents) {
							if (itemSectionContent.gridRenderer !== undefined) {
								const playlistResults = []
								for (const item of itemSectionContent.gridRenderer.items) {
									if (item.gridPlaylistRenderer !== undefined) {
										playlistResults.push({
											kind: 'playlist',
											subtitleOrNull: channelTitle,
											title: YtTextAsStringId(item.gridPlaylistRenderer.title),
											value: StringId(item.gridPlaylistRenderer.playlistId),
										})
									}
								}
								results.push({
									kind: 'channelPlaylists',
									subtitleOrNull: channelTitle,
									title: StringId((
										sectionListRenderer
										.subMenu
										.channelSubMenuRenderer
										.contentTypeSubMenuItems
										.find((item) => {
											return item.selected
										})
										.title
									)),
									value: playlistResults,
								})
							} else if (itemSectionContent.shelfRenderer !== undefined) {
								const shelfRenderer = itemSectionContent.shelfRenderer
								const content = shelfRenderer.content
								if (content.expandedShelfContentsRenderer !== undefined) {
									results.push({
										kind: 'channelPlaylists',
										subtitleOrNull: channelTitle,
										title: YtTextAsStringId(shelfRenderer.title),
										value: content.expandedShelfContentsRenderer.items.map((item) => {
											return {
												kind: 'playlist',
												subtitleOrNull: channelTitle,
												title: YtTextAsStringId(item.playlistRenderer.title),
												value: StringId(item.playlistRenderer.playlistId),
											}
										}),
									})
								} else if (content.horizontalListRenderer !== undefined) {
									results.push({
										kind: 'channelPlaylists',
										subtitleOrNull: channelTitle,
										title: YtTextAsStringId(shelfRenderer.title),
										value: content.horizontalListRenderer.items.map((item) => {
											return {
												kind: 'playlist',
												subtitleOrNull: channelTitle,
												title: YtTextAsStringId(item.gridPlaylistRenderer.title),
												value: StringId(item.gridPlaylistRenderer.playlistId),
											}
										}),
									})
								}
							}
						}
					}
				}
			}
		}
		if (tabs.some((tab) => {
			if (tab.tabRenderer === undefined) {
				return false
			}
			const splitPathname = RelativeUriAsUrl((
				tab
				.tabRenderer
				.endpoint
				.commandMetadata
				.webCommandMetadata
				.url
			)).pathname.split('/')
			return splitPathname[splitPathname.length - 1] === 'releases'
		})) {
			results.push({
				kind: 'channelReleases',
				title: 'Releases',
				subtitleOrNull: channelTitle,
				value: channel,
			})
		}
	} catch (throwee) {
		threw = true
		throweeIfThrew = throwee
	}
	qsWriteDebuggingInfo('qsYtChannelResultsOrNullInfo', {
		channel: channel,
		channelTitle: channelTitle,
		resBodyOrNull: resBodyOrNull === null ? null : resBodyOrNull.toString('base64'),
		resOrNull: resOrNull === null ? null : HttpResAsJsonValue(resOrNull),
		threw: threw,
		throweeIfThrew: threw ? ThroweeAsJsonValue(throweeIfThrew) : null,
	})
	return threw ? null : results
}

export default QsYtChannelResultsOrNull
