import bufferHttpResBody from './bufferHttpResBody.js'
import HttpResAsJsonValue from './HttpResAsJsonValue.js'
import HttpsGetRes from './HttpsGetRes.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import QueryString from './QueryString.js'
import StringId from './StringId.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'
import Ungzipped from './Ungzipped.js'
import YtInitialData from './YtInitialData.js'
import YtResultDuration from './YtResultDuration.js'
import YtTextAsStringId from './YtTextAsStringId.js'

const QsYtPlaylistVideoResultsOrNull = async (playlist) => {
	let resOrNull = null
	let resBodyOrNull = null
	const results = []
	let threw = false
	let throweeIfThrew
	try {
		resOrNull = await HttpsGetRes(
			'https://www.youtube.com/watch?' + QueryString({list: playlist}),
			{headers: {'Accept-Encoding': 'gzip', 'Accept-Language': 'en-US'}},
		)
		resBodyOrNull = await bufferHttpResBody(resOrNull)
		for (const content of (
			YtInitialData((await Ungzipped(resBodyOrNull)).toString())
			.contents
			.twoColumnWatchNextResults
			.playlist
			.playlist
			.contents
		)) {
			if (content.playlistPanelVideoRenderer !== undefined) {
				const videoRenderer = content.playlistPanelVideoRenderer
				if (videoRenderer.lengthText !== undefined) {
					results.push({
						kind: 'video',
						subtitleOrNull: YtTextAsStringId(videoRenderer.longBylineText),
						title: YtTextAsStringId(videoRenderer.title),
						value: {duration: YtResultDuration(videoRenderer.lengthText), video: StringId(videoRenderer.videoId)},
					})
				}
			}
		}
	} catch (throwee) {
		threw = true
		throweeIfThrew = throwee
	}
	qsWriteDebuggingInfo('qsYtPlaylistVideoResultsOrNullInfo', {
		playlist: playlist,
		resBodyOrNull: resBodyOrNull === null ? null : resBodyOrNull.toString('base64'),
		resOrNull: resOrNull === null ? null : HttpResAsJsonValue(resOrNull),
		threw: threw,
		throweeIfThrew: threw ? ThroweeAsJsonValue(throweeIfThrew) : null,
	})
	return threw ? null : results
}

export default QsYtPlaylistVideoResultsOrNull
