import bufferHttpResBody from './bufferHttpResBody.js'
import HttpResAsJsonValue from './HttpResAsJsonValue.js'
import HttpsGetRes from './HttpsGetRes.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import QueryString from './QueryString.js'
import RelativeUriAsUrl from './RelativeUriAsUrl.js'
import StringId from './StringId.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'
import Ungzipped from './Ungzipped.js'
import YtInitialData from './YtInitialData.js'
import YtResultDuration from './YtResultDuration.js'
import YtTextAsStringId from './YtTextAsStringId.js'

const QsYtSearchResultsOrNull = async (query) => {
	let resOrNull = null
	let resBodyOrNull = null
	const results = []
	let threw = false
	let throweeIfThrew
	try {
		resOrNull = await HttpsGetRes(
			'https://www.youtube.com/results?' + QueryString({search_query: query}),
			{headers: {'Accept-Encoding': 'gzip', 'Accept-Language': 'en-US'}},
		)
		resBodyOrNull = await bufferHttpResBody(resOrNull)
		const twoColumnSearchResultsRenderer = (
			YtInitialData((await Ungzipped(resBodyOrNull)).toString())
			.contents
			.twoColumnSearchResultsRenderer
		)
		const cardChannelsInResults = new Set()
		const cardPlaylistsInResults = new Set()
		if (twoColumnSearchResultsRenderer.secondaryContents !== undefined) {
			for (const content of (
				twoColumnSearchResultsRenderer
				.secondaryContents
				.secondarySearchContainerRenderer
				.contents
			)) {
				const watchCardRichHeaderRenderer = (
					content
					.universalWatchCardRenderer
					.header
					.watchCardRichHeaderRenderer
				)
				const webCommandMetadata = (
					watchCardRichHeaderRenderer
					.titleNavigationEndpoint
					.commandMetadata
					.webCommandMetadata
				)
				if (webCommandMetadata.webPageType === 'WEB_PAGE_TYPE_CHANNEL') {
					const channel = StringId((
						watchCardRichHeaderRenderer
						.titleNavigationEndpoint
						.browseEndpoint
						.browseId
					))
					results.push({
						kind: 'channel',
						subtitleOrNull: null,
						title: YtTextAsStringId(watchCardRichHeaderRenderer.title),
						value: channel,
					})
					cardChannelsInResults.add(channel)
				} else if (webCommandMetadata.webPageType === 'WEB_PAGE_TYPE_PLAYLIST') {
					const playlist = StringId(RelativeUriAsUrl(webCommandMetadata.url).searchParams.get('list'))
					results.push({
						kind: 'playlist',
						subtitleOrNull: YtTextAsStringId(watchCardRichHeaderRenderer.subtitle),
						title: YtTextAsStringId(watchCardRichHeaderRenderer.title),
						value: playlist,
					})
					cardPlaylistsInResults.add(playlist)
				}
			}
		}
		for (const sectionListContent of (
			twoColumnSearchResultsRenderer
			.primaryContents
			.sectionListRenderer
			.contents
		)) {
			if (sectionListContent.itemSectionRenderer !== undefined) {
				for (const content of sectionListContent.itemSectionRenderer.contents) {
					if (content.channelRenderer !== undefined) {
						const channelRenderer = content.channelRenderer
						const channel = StringId(channelRenderer.channelId)
						if (!cardChannelsInResults.has(channel)) {
							results.push({
								kind: 'channel',
								subtitleOrNull: null,
								title: YtTextAsStringId(channelRenderer.title),
								value: channel,
							})
						}
					} else if (content.playlistRenderer !== undefined) {
						const playlistRenderer = content.playlistRenderer
						const playlist = StringId(playlistRenderer.playlistId)
						if (!cardPlaylistsInResults.has(playlist)) {
							results.push({
								kind: 'playlist',
								subtitleOrNull: YtTextAsStringId(playlistRenderer.longBylineText),
								title: YtTextAsStringId(playlistRenderer.title),
								value: playlist,
							})
						}
					} else if (content.videoRenderer !== undefined) {
						const videoRenderer = content.videoRenderer
						if (videoRenderer.lengthText !== undefined) {
							results.push({
								kind: 'video',
								subtitleOrNull: YtTextAsStringId(videoRenderer.longBylineText),
								title: YtTextAsStringId(videoRenderer.title),
								value: {duration: YtResultDuration(videoRenderer.lengthText), video: StringId(videoRenderer.videoId)},
							})
						}
					}
				}
			}
		}
	} catch (throwee) {
		threw = true
		throweeIfThrew = throwee
	}
	qsWriteDebuggingInfo('qsYtSearchResultsOrNullInfo', {
		query: query,
		resBodyOrNull: resBodyOrNull === null ? null : resBodyOrNull.toString('base64'),
		resOrNull: resOrNull === null ? null : HttpResAsJsonValue(resOrNull),
		threw: threw,
		throweeIfThrew: threw ? ThroweeAsJsonValue(throweeIfThrew) : null,
	})
	return threw ? null : results
}

export default QsYtSearchResultsOrNull
