import QcQueueGroups from './QcQueueGroups.js'
import ArrayLastValue from './ArrayLastValue.js'
import QcQueuePlayInfo from './QcQueuePlayInfo.js'
import assert from './assert.js'
import qcSkipBackMap from './qcSkipBackMap.js'
import QcSkipBackPositions from './QcSkipBackPositions.js'
import qcQueueMap from './qcQueueMap.js'
import qcsCollabMap from './qcsCollabMap.js'
import QcQueuePositions from './QcQueuePositions.js'
import QcQueueIsPlaying from './QcQueueIsPlaying.js'
import CollabPosition from './CollabPosition.js'

const skipBackPlaylist = (queuePositions, playlist) => {
	const skipBackValues = QcSkipBackPositions().map((position) => {
		return qcSkipBackMap.get(position)
	})
	for (let i = skipBackValues.length - 1; i >= 0; i--) {
		const previousPlaylistOrUndefined = skipBackValues[i].playlist
		if (playlist !== previousPlaylistOrUndefined) {
			const previousTrack = skipBackValues[i].track
			const skipBackInfo = JSON.parse(qcsCollabMap.get(previousPlaylistOrUndefined === undefined ? 'media-' + previousTrack + '-skipBackInfo' : 'media-' + previousTrack + '-playlists-' + previousPlaylistOrUndefined + '-skipBackInfo'))
			assert(queuePositions.length !== 0)
			qcsCollabMap.delete(previousPlaylistOrUndefined === undefined ? 'media-' + previousTrack + '-skipBackInfo' : 'media-' + previousTrack + '-playlists-' + previousPlaylistOrUndefined + '-skipBackInfo')
			qcsCollabMap.set(
				previousPlaylistOrUndefined === undefined ? 'media-' + previousTrack + '-queueInfo' : 'media-' + previousTrack + '-playlists-' + previousPlaylistOrUndefined + '-queueInfo',
				JSON.stringify({...skipBackInfo, position: CollabPosition(null, queuePositions[0])}),
			)
			return true
		}
	}
	return false
}
const qcUnbumpPlaylist = (playlist) => {
	const positions = QcQueuePositions()
	const firstIndexOrNegativeOne = positions.findIndex((position) => {
		return qcQueueMap.get(position).playlist === playlist
	})
	if (firstIndexOrNegativeOne !== -1) {
		const firstIndex = firstIndexOrNegativeOne
		let unbumped = false
		if (firstIndex === 0 && QcQueueIsPlaying()) {
			const playInfo = QcQueuePlayInfo()
			if (playInfo.sourcePlaylist === playlist) {
				QcQueueIsPlaying.set(false)
				unbumped = true
			} else {
				unbumped = skipBackPlaylist(positions, playlist)
			}
		}
		if (!unbumped) {
			const groups = QcQueueGroups()
			const firstGroupIndex = groups.findIndex((group) => {
				return qcQueueMap.get(group[0]).playlist === playlist
			})
			assert(firstGroupIndex !== -1)
			const tracksInPlaylist = positions.filter((position) => {
				return qcQueueMap.get(position).playlist === playlist
			}).map((position) => {
				return qcQueueMap.get(position).track
			})
			if (firstGroupIndex === groups.length - 1) {
				for (const track of tracksInPlaylist) {
					qcsCollabMap.delete('media-' + track + '-playlists-' + playlist + '-queueInfo')
				}
			} else {
				let afterPosition = ArrayLastValue(groups[firstGroupIndex + 1])
				const beforePositionOrNull = firstGroupIndex === groups.length - 2 ? null : groups[firstGroupIndex + 2][0]
				for (const track of tracksInPlaylist) {
					const newPosition = CollabPosition(afterPosition, beforePositionOrNull)
					qcsCollabMap.set(
						'media-' + track + '-playlists-' + playlist + '-queueInfo',
						JSON.stringify({position: newPosition}),
					)
					afterPosition = newPosition
				}
			}
		}
	}
}

export default qcUnbumpPlaylist
