import qcsCollabMap from './qcsCollabMap.js'
import qcSetupArg from './qcSetupArg.js'
import QueryString from './QueryString.js'
import Queue from './Queue.js'
import wait from './wait.js'

import './qcCollabMapSetup.js'

let qsCollabMapTime = qcSetupArg.qsCollabMapTime
const qsCollabThread = qcSetupArg.qsCollabThread
let currentWs
let currentWsUnconfirmedCollabMapOps
const collabMapOpsToSync = new Map()
;(async () => {
	while (true) {
		const ws = new WebSocket('ws://' + location.host + '/qcWs?' + QueryString({
			qsCollabMapTime: JSON.stringify(qsCollabMapTime),
			qsCollabThread: qsCollabThread,
		}))
		currentWs = ws
		const wsUnconfirmedCollabMapOps = new Queue()
		currentWsUnconfirmedCollabMapOps = wsUnconfirmedCollabMapOps
		await new Promise((resolve) => {
			setTimeout(() => {
				if (ws.readyState === 0) {
					ws.close()
					resolve()
				}
			}, 4000)
			let wsOpened = false
			ws.addEventListener('open', () => {
				wsOpened = true
				for (const op of collabMapOpsToSync.values()) {
					ws.send(JSON.stringify({kind: 'collabMapOp', value: op}))
					wsUnconfirmedCollabMapOps.append(op)
				}
				let wsHasReceivedMessageSinceLastPing = false
				;(async () => {
					while (true) {
						ws.send(JSON.stringify({kind: 'ping', value: null}))
						await wait(4000)
						if (!wsHasReceivedMessageSinceLastPing) {
							ws.close()
							resolve()
							break
						}
						wsHasReceivedMessageSinceLastPing = false
					}
				})()
				ws.addEventListener('message', (event) => {
					wsHasReceivedMessageSinceLastPing = true
					const message = JSON.parse(event.data)
					const kind = message.kind
					const value = message.value
					if (kind === 'collabMapCatchUpOp') {
						qcsCollabMap.receive(value)
					} else if (kind === 'collabMapCatchUpTime') {
						qsCollabMapTime = value
					} else if (kind === 'collabMapOp') {
						qcsCollabMap.receive(value)
						qsCollabMapTime++
					} else if (kind === 'collabMapOpConfirmation') {
						const op = wsUnconfirmedCollabMapOps.firstValue()
						wsUnconfirmedCollabMapOps.deleteFirstValue()
						if (collabMapOpsToSync.get(op.key) === op) {
							collabMapOpsToSync.delete(op.key)
						}
					} else if (kind === 'qsCollabThreadMismatch') {
						location.reload()
					}
				}, {passive: true})
			}, {passive: true})
			ws.addEventListener('close', () => {
				if (wsOpened) {
					resolve()
				} else {
					setTimeout(resolve, 1000)
				}
			}, {passive: true})
		})
	}
})()
const handleNewCollabMapOp = (op) => {
	if (qcsCollabMap.localOps.has(op)) {
		collabMapOpsToSync.set(op.key, op)
		if (currentWs.readyState === 1) {
			currentWs.send(JSON.stringify({kind: 'collabMapOp', value: op}))
			currentWsUnconfirmedCollabMapOps.append(op)
		}
	}
}
for (const op of qcsCollabMap.ops.values()) {
	handleNewCollabMapOp(op)
}
qcsCollabMap.changeHandlers.push((change) => {
	handleNewCollabMapOp(change.op)
})
