import CollabPositionComparison from './CollabPositionComparison.js'
import CollabMap from './CollabMap.js'
import CollabPosition from './CollabPosition.js'
import fs from 'node:fs'
import FsPath from 'node:path'
import processNewlineDelimitedFile from './processNewlineDelimitedFile.js'
import qcsCollabMap from './qcsCollabMap.js'
// import QcsMediaExists from './QcsMediaExists.js'
// import QcsPlaylistMaps from './QcsPlaylistMaps.js'
import qsDataDir from './qsDataDir.js'
// import QsPlaylistSortedMedia from './QsPlaylistSortedMedia.js'

const opsFile = FsPath.join(qsDataDir, 'collab.ndjson')
await (async () => {
	const tempCollabMap = new CollabMap()
	await fs.promises.appendFile(opsFile, '')
	await processNewlineDelimitedFile(opsFile, (opAsJson) => {
		tempCollabMap.receive(JSON.parse(opAsJson))
	})
	for (const [key, value] of tempCollabMap) {
		qcsCollabMap.set(key, value)
	}
	{
		// mediaSources -> media
		for (const [key, value] of [...qcsCollabMap]) {
			const matchOrNull = key.match(/^media-([^-]*)-info$/)
			if (matchOrNull !== null) {
				const [, media] = matchOrNull
				const source = JSON.parse(value).source
				qcsCollabMap.set('media-' + media + '-title', qcsCollabMap.get('mediaSources-' + source + '-title'))
				qcsCollabMap.set('media-' + media + '-subtitle', qcsCollabMap.get('mediaSources-' + source + '-subtitle'))
				qcsCollabMap.set('media-' + media + '-duration', qcsCollabMap.get('mediaSources-' + source + '-duration'))
				qcsCollabMap.set('media-' + media + '-info2', qcsCollabMap.get('mediaSources-' + source + '-info'))
				qcsCollabMap.delete(key)
			}
		}
		for (const key of [...qcsCollabMap.keys()]) {
			if (/^mediaSources-/.test(key)) {
				qcsCollabMap.delete(key)
			}
		}
	}
	{
		// recentsPosition -> recentsInfo
		const time = Date.now()
		for (const [key, value] of [...qcsCollabMap]) {
			const matchOrNull = key.match(/^media-([^-]*)-recentsPosition$/)
			if (matchOrNull !== null) {
				const [, media] = matchOrNull
				qcsCollabMap.set('media-' + media + '-recentsInfo', JSON.stringify({position: value, time: time}))
				qcsCollabMap.delete(key)
			}
		}
	}
	qcsCollabMap.delete('queueIsPlaying')
	{
		// delete trackQueueInfo.bumpStartedPlayback
		for (const [key, value] of [...qcsCollabMap]) {
			if (/^media-[^-]*-queueInfo$/.test(key)) {
				qcsCollabMap.set(key, JSON.stringify({...JSON.parse(value), bumpStartedPlayback: undefined}))
			}
		}
	}
	{
		// compress recents positions
		const map = new Map()
		for (const [key, value] of qcsCollabMap) {
			const matchOrNull = key.match(/^media-([^-]*)-recentsInfo$/)
			if (matchOrNull !== null) {
				const [, media] = matchOrNull
				const info = JSON.parse(value)
				map.set(info.position, {media: media, info: info})
			}
		}
		const positions = [...map.keys()]
		positions.sort(CollabPositionComparison)
		let previousCompressedPositionOrNull = null
		for (const position of positions) {
			const compressedPosition = CollabPosition(previousCompressedPositionOrNull, null)
			qcsCollabMap.set('media-' + map.get(position).media + '-recentsInfo', JSON.stringify({...map.get(position).info, position: compressedPosition}))
			previousCompressedPositionOrNull = compressedPosition
		}
	}
	{
		// compress skipBack positions
		const map = new Map()
		for (const [key, value] of qcsCollabMap) {
			const matchOrNull = key.match(/^media-([^-]*)-skipBackInfo$/)
			if (matchOrNull !== null) {
				const [, track] = matchOrNull
				const info = JSON.parse(value)
				map.set(info.position, {track: track, info: info})
			}
		}
		for (const [key, value] of qcsCollabMap) {
			const matchOrNull = key.match(/^media-([^-]*)-playlists-([^-]*)-skipBackInfo$/)
			if (matchOrNull !== null) {
				const [, track, playlist] = matchOrNull
				const info = JSON.parse(value)
				map.set(info.position, {track: track, playlist: playlist, info: info})
			}
		}
		const positions = [...map.keys()]
		positions.sort(CollabPositionComparison)
		let previousCompressedPositionOrNull = null
		for (const position of positions) {
			const compressedPosition = CollabPosition(previousCompressedPositionOrNull, null)
			const value = map.get(position)
			qcsCollabMap.set(
				value.playlist === undefined ? 'media-' + value.track + '-skipBackInfo' : 'media-' + value.track + '-playlists-' + value.playlist + '-skipBackInfo',
				JSON.stringify({...value.info, position: compressedPosition}),
			)
			previousCompressedPositionOrNull = compressedPosition
		}
	}
	// const QcsPlaylistMaps = () => {
	// 	const maps = new Map()
	// 	for (const key of qcsCollabMap.ops.keys()) {
	// 		const matchOrNull = key.match(/^media-([^-]*)-playlists-([^-]*)-info$/)
	// 		if (matchOrNull !== null) {
	// 			const value = qcsCollabMap.ops.get(key).value
	// 			if (value !== '') {
	// 				const [, media, playlist] = matchOrNull
	// 				if (!maps.has(playlist)) {
	// 					maps.set(playlist, new Map())
	// 				}
	// 				maps.get(playlist).set(JSON.parse(value).position, media)
	// 			}
	// 		}
	// 	}
	// 	return maps
	// }
	// ;(() => {
	// 	const playlistMaps = QcsPlaylistMaps()
	// 	for (const playlist of playlistMaps.keys()) {
	// 		let previousCompressedPositionOrNull = null
	// 		for (const media of QsPlaylistSortedMedia(playlistMaps, playlist)) {
	// 			const infoKey = 'media-' + media + '-playlists-' + playlist + '-info'
	// 			const compressedPosition = CollabPosition(previousCompressedPositionOrNull, null)
	// 			qcsCollabMap.set(infoKey, JSON.stringify({
	// 				...JSON.parse(qcsCollabMap.get(infoKey)),
	// 				position: compressedPosition,
	// 			}))
	// 			previousCompressedPositionOrNull = compressedPosition
	// 		}
	// 	}
	// })()
	// ;(() => {
	// 	const recentsSortedMedia = QsPlaylistSortedMedia(QcsPlaylistMaps(), 'recents')
	// 	let liveRecentsSize = recentsSortedMedia.length
	// 	for (let i = recentsSortedMedia.length - 1; i >= 0; i--) {
	// 		if (liveRecentsSize <= 10000) {
	// 			break
	// 		}
	// 		qcsCollabMap.delete('media-' + recentsSortedMedia[i] + '-playlists-recents-info')
	// 		liveRecentsSize--
	// 	}
	// })()
	// for (const key of [...qcsCollabMap.keys()]) {
	// 	const matchOrNull = key.match(/^media-([^-]*)/)
	// 	if (matchOrNull !== null) {
	// 		const [, media] = matchOrNull
	// 		if (!QcsMediaExists(media)) {
	// 			qcsCollabMap.delete(key)
	// 		}
	// 	}
	// }
	// for (const key of [...qcsCollabMap.keys()]) {
	// 	const matchOrNull = key.match(/^media-[^-]*-playlists-([^-]*)/)
	// 	if (matchOrNull !== null) {
	// 		const [, playlist] = matchOrNull
	// 		if (!QcsMediaExists(playlist)) {
	// 			qcsCollabMap.delete(key)
	// 		}
	// 	}
	// }
	// for (const key of [...qcsCollabMap.keys()]) {
	// 	const matchOrNull = key.match(/^media-([^-]*)-playlists-([^-]*)/)
	// 	if (matchOrNull !== null) {
	// 		const [, media, playlist] = matchOrNull
	// 		if (!qcsCollabMap.has('media-' + media + '-playlists-' + playlist + '-info')) {
	// 			qcsCollabMap.delete(key)
	// 		}
	// 	}
	// }
	// ;(() => {
	// 	const playlistMaps = QcsPlaylistMaps()
	// 	const reachableMedia = new Set(['queue', 'recents'])
	// 	for (const track of QsPlaylistSortedMedia(playlistMaps, 'queue')) {
	// 		reachableMedia.add(track)
	// 		const queuedPlaylistOrNull = JSON.parse(qcsCollabMap.get('media-' + track + '-playlists-queue-info')).queuedPlaylistOrNull
	// 		if (queuedPlaylistOrNull !== null && !reachableMedia.has(queuedPlaylistOrNull)) {
	// 			reachableMedia.add(queuedPlaylistOrNull)
	// 			for (const trackInQueuedPlaylist of QsPlaylistSortedMedia(playlistMaps, queuedPlaylistOrNull)) {
	// 				reachableMedia.add(trackInQueuedPlaylist)
	// 			}
	// 		}
	// 	}
	// 	for (const media of QsPlaylistSortedMedia(playlistMaps, 'recents')) {
	// 		reachableMedia.add(media)
	// 		for (const track of QsPlaylistSortedMedia(playlistMaps, media)) {
	// 			reachableMedia.add(track)
	// 		}
	// 	}
	// 	for (const key of [...qcsCollabMap.keys()]) {
	// 		const matchOrNull = key.match(/^media-([^-]*)/)
	// 		if (matchOrNull !== null) {
	// 			const [, media] = matchOrNull
	// 			if (!reachableMedia.has(media)) {
	// 				qcsCollabMap.delete(key)
	// 			}
	// 		}
	// 	}
	// 	for (const key of [...qcsCollabMap.keys()]) {
	// 		const matchOrNull = key.match(/^media-[^-]*-playlists-([^-]*)/)
	// 		if (matchOrNull !== null) {
	// 			const [, playlist] = matchOrNull
	// 			if (!reachableMedia.has(playlist)) {
	// 				qcsCollabMap.delete(key)
	// 			}
	// 		}
	// 	}
	// })()
})()
const tempOpsFile = FsPath.join(qsDataDir, 'temp_collab.ndjson')
let newOpsFileContents = ''
for (const op of qcsCollabMap.ops.values()) {
	newOpsFileContents += JSON.stringify(op) + '\n'
}
fs.writeFileSync(tempOpsFile, newOpsFileContents)
fs.renameSync(tempOpsFile, opsFile)
const opsFileAppendStream = fs.createWriteStream(opsFile, {flags: 'a'})
qcsCollabMap.changeHandlers.push((change) => {
	opsFileAppendStream.write(JSON.stringify(change.op) + '\n')
})
